from ._core import (
    DataRecord,
    DataRecordGraphQL,
    DataRecordWrite,
    DomainModel,
    DomainModelCore,
    DomainModelWrite,
    DomainModelList,
    DomainRelationWrite,
    GraphQLCore,
    GraphQLList,
    ResourcesWrite,
    ResourcesWriteResult,
    PageInfo,
)
from ._alert import (
    Alert,
    AlertApply,
    AlertApplyList,
    AlertFields,
    AlertGraphQL,
    AlertList,
    AlertTextFields,
    AlertWrite,
    AlertWriteList,
)
from ._benchmarking_calculation_input import (
    BenchmarkingCalculationInput,
    BenchmarkingCalculationInputApply,
    BenchmarkingCalculationInputApplyList,
    BenchmarkingCalculationInputFields,
    BenchmarkingCalculationInputGraphQL,
    BenchmarkingCalculationInputList,
    BenchmarkingCalculationInputTextFields,
    BenchmarkingCalculationInputWrite,
    BenchmarkingCalculationInputWriteList,
)
from ._benchmarking_calculation_output import (
    BenchmarkingCalculationOutput,
    BenchmarkingCalculationOutputApply,
    BenchmarkingCalculationOutputApplyList,
    BenchmarkingCalculationOutputFields,
    BenchmarkingCalculationOutputGraphQL,
    BenchmarkingCalculationOutputList,
    BenchmarkingCalculationOutputTextFields,
    BenchmarkingCalculationOutputWrite,
    BenchmarkingCalculationOutputWriteList,
)
from ._benchmarking_configuration_day_ahead import (
    BenchmarkingConfigurationDayAhead,
    BenchmarkingConfigurationDayAheadApply,
    BenchmarkingConfigurationDayAheadApplyList,
    BenchmarkingConfigurationDayAheadFields,
    BenchmarkingConfigurationDayAheadGraphQL,
    BenchmarkingConfigurationDayAheadList,
    BenchmarkingConfigurationDayAheadTextFields,
    BenchmarkingConfigurationDayAheadWrite,
    BenchmarkingConfigurationDayAheadWriteList,
)
from ._benchmarking_production_obligation_day_ahead import (
    BenchmarkingProductionObligationDayAhead,
    BenchmarkingProductionObligationDayAheadApply,
    BenchmarkingProductionObligationDayAheadApplyList,
    BenchmarkingProductionObligationDayAheadFields,
    BenchmarkingProductionObligationDayAheadGraphQL,
    BenchmarkingProductionObligationDayAheadList,
    BenchmarkingProductionObligationDayAheadTextFields,
    BenchmarkingProductionObligationDayAheadWrite,
    BenchmarkingProductionObligationDayAheadWriteList,
)
from ._benchmarking_result_day_ahead import (
    BenchmarkingResultDayAhead,
    BenchmarkingResultDayAheadApply,
    BenchmarkingResultDayAheadApplyList,
    BenchmarkingResultDayAheadFields,
    BenchmarkingResultDayAheadGraphQL,
    BenchmarkingResultDayAheadList,
    BenchmarkingResultDayAheadTextFields,
    BenchmarkingResultDayAheadWrite,
    BenchmarkingResultDayAheadWriteList,
)
from ._benchmarking_shop_case import (
    BenchmarkingShopCase,
    BenchmarkingShopCaseApply,
    BenchmarkingShopCaseApplyList,
    BenchmarkingShopCaseFields,
    BenchmarkingShopCaseGraphQL,
    BenchmarkingShopCaseList,
    BenchmarkingShopCaseWrite,
    BenchmarkingShopCaseWriteList,
)
from ._benchmarking_task_dispatcher_input_day_ahead import (
    BenchmarkingTaskDispatcherInputDayAhead,
    BenchmarkingTaskDispatcherInputDayAheadApply,
    BenchmarkingTaskDispatcherInputDayAheadApplyList,
    BenchmarkingTaskDispatcherInputDayAheadFields,
    BenchmarkingTaskDispatcherInputDayAheadGraphQL,
    BenchmarkingTaskDispatcherInputDayAheadList,
    BenchmarkingTaskDispatcherInputDayAheadTextFields,
    BenchmarkingTaskDispatcherInputDayAheadWrite,
    BenchmarkingTaskDispatcherInputDayAheadWriteList,
)
from ._benchmarking_task_dispatcher_output_day_ahead import (
    BenchmarkingTaskDispatcherOutputDayAhead,
    BenchmarkingTaskDispatcherOutputDayAheadApply,
    BenchmarkingTaskDispatcherOutputDayAheadApplyList,
    BenchmarkingTaskDispatcherOutputDayAheadFields,
    BenchmarkingTaskDispatcherOutputDayAheadGraphQL,
    BenchmarkingTaskDispatcherOutputDayAheadList,
    BenchmarkingTaskDispatcherOutputDayAheadTextFields,
    BenchmarkingTaskDispatcherOutputDayAheadWrite,
    BenchmarkingTaskDispatcherOutputDayAheadWriteList,
)
from ._bid_configuration_day_ahead import (
    BidConfigurationDayAhead,
    BidConfigurationDayAheadApply,
    BidConfigurationDayAheadApplyList,
    BidConfigurationDayAheadFields,
    BidConfigurationDayAheadGraphQL,
    BidConfigurationDayAheadList,
    BidConfigurationDayAheadTextFields,
    BidConfigurationDayAheadWrite,
    BidConfigurationDayAheadWriteList,
)
from ._bid_document import (
    BidDocument,
    BidDocumentApply,
    BidDocumentApplyList,
    BidDocumentFields,
    BidDocumentGraphQL,
    BidDocumentList,
    BidDocumentTextFields,
    BidDocumentWrite,
    BidDocumentWriteList,
)
from ._bid_document_afrr import (
    BidDocumentAFRR,
    BidDocumentAFRRApply,
    BidDocumentAFRRApplyList,
    BidDocumentAFRRFields,
    BidDocumentAFRRGraphQL,
    BidDocumentAFRRList,
    BidDocumentAFRRTextFields,
    BidDocumentAFRRWrite,
    BidDocumentAFRRWriteList,
)
from ._bid_document_day_ahead import (
    BidDocumentDayAhead,
    BidDocumentDayAheadApply,
    BidDocumentDayAheadApplyList,
    BidDocumentDayAheadFields,
    BidDocumentDayAheadGraphQL,
    BidDocumentDayAheadList,
    BidDocumentDayAheadTextFields,
    BidDocumentDayAheadWrite,
    BidDocumentDayAheadWriteList,
)
from ._bid_matrix import (
    BidMatrix,
    BidMatrixApply,
    BidMatrixApplyList,
    BidMatrixFields,
    BidMatrixGraphQL,
    BidMatrixList,
    BidMatrixTextFields,
    BidMatrixWrite,
    BidMatrixWriteList,
)
from ._bid_matrix_information import (
    BidMatrixInformation,
    BidMatrixInformationApply,
    BidMatrixInformationApplyList,
    BidMatrixInformationFields,
    BidMatrixInformationGraphQL,
    BidMatrixInformationList,
    BidMatrixInformationTextFields,
    BidMatrixInformationWrite,
    BidMatrixInformationWriteList,
)
from ._bid_row import (
    BidRow,
    BidRowApply,
    BidRowApplyList,
    BidRowFields,
    BidRowGraphQL,
    BidRowList,
    BidRowTextFields,
    BidRowWrite,
    BidRowWriteList,
)
from ._date_specification import (
    DateSpecification,
    DateSpecificationApply,
    DateSpecificationApplyList,
    DateSpecificationFields,
    DateSpecificationGraphQL,
    DateSpecificationList,
    DateSpecificationTextFields,
    DateSpecificationWrite,
    DateSpecificationWriteList,
)
from ._function_input import (
    FunctionInput,
    FunctionInputApply,
    FunctionInputApplyList,
    FunctionInputFields,
    FunctionInputGraphQL,
    FunctionInputList,
    FunctionInputTextFields,
    FunctionInputWrite,
    FunctionInputWriteList,
)
from ._function_output import (
    FunctionOutput,
    FunctionOutputApply,
    FunctionOutputApplyList,
    FunctionOutputFields,
    FunctionOutputGraphQL,
    FunctionOutputList,
    FunctionOutputTextFields,
    FunctionOutputWrite,
    FunctionOutputWriteList,
)
from ._generator import (
    Generator,
    GeneratorApply,
    GeneratorApplyList,
    GeneratorFields,
    GeneratorGraphQL,
    GeneratorList,
    GeneratorTextFields,
    GeneratorWrite,
    GeneratorWriteList,
)
from ._generator_efficiency_curve import (
    GeneratorEfficiencyCurve,
    GeneratorEfficiencyCurveApply,
    GeneratorEfficiencyCurveApplyList,
    GeneratorEfficiencyCurveFields,
    GeneratorEfficiencyCurveGraphQL,
    GeneratorEfficiencyCurveList,
    GeneratorEfficiencyCurveWrite,
    GeneratorEfficiencyCurveWriteList,
)
from ._market_configuration import (
    MarketConfiguration,
    MarketConfigurationApply,
    MarketConfigurationApplyList,
    MarketConfigurationFields,
    MarketConfigurationGraphQL,
    MarketConfigurationList,
    MarketConfigurationTextFields,
    MarketConfigurationWrite,
    MarketConfigurationWriteList,
)
from ._multi_scenario_partial_bid_matrix_calculation_input import (
    MultiScenarioPartialBidMatrixCalculationInput,
    MultiScenarioPartialBidMatrixCalculationInputApply,
    MultiScenarioPartialBidMatrixCalculationInputApplyList,
    MultiScenarioPartialBidMatrixCalculationInputFields,
    MultiScenarioPartialBidMatrixCalculationInputGraphQL,
    MultiScenarioPartialBidMatrixCalculationInputList,
    MultiScenarioPartialBidMatrixCalculationInputTextFields,
    MultiScenarioPartialBidMatrixCalculationInputWrite,
    MultiScenarioPartialBidMatrixCalculationInputWriteList,
)
from ._partial_bid_configuration import (
    PartialBidConfiguration,
    PartialBidConfigurationApply,
    PartialBidConfigurationApplyList,
    PartialBidConfigurationFields,
    PartialBidConfigurationGraphQL,
    PartialBidConfigurationList,
    PartialBidConfigurationTextFields,
    PartialBidConfigurationWrite,
    PartialBidConfigurationWriteList,
)
from ._partial_bid_matrix_calculation_input import (
    PartialBidMatrixCalculationInput,
    PartialBidMatrixCalculationInputApply,
    PartialBidMatrixCalculationInputApplyList,
    PartialBidMatrixCalculationInputFields,
    PartialBidMatrixCalculationInputGraphQL,
    PartialBidMatrixCalculationInputList,
    PartialBidMatrixCalculationInputTextFields,
    PartialBidMatrixCalculationInputWrite,
    PartialBidMatrixCalculationInputWriteList,
)
from ._partial_bid_matrix_calculation_output import (
    PartialBidMatrixCalculationOutput,
    PartialBidMatrixCalculationOutputApply,
    PartialBidMatrixCalculationOutputApplyList,
    PartialBidMatrixCalculationOutputFields,
    PartialBidMatrixCalculationOutputGraphQL,
    PartialBidMatrixCalculationOutputList,
    PartialBidMatrixCalculationOutputTextFields,
    PartialBidMatrixCalculationOutputWrite,
    PartialBidMatrixCalculationOutputWriteList,
)
from ._partial_bid_matrix_information import (
    PartialBidMatrixInformation,
    PartialBidMatrixInformationApply,
    PartialBidMatrixInformationApplyList,
    PartialBidMatrixInformationFields,
    PartialBidMatrixInformationGraphQL,
    PartialBidMatrixInformationList,
    PartialBidMatrixInformationTextFields,
    PartialBidMatrixInformationWrite,
    PartialBidMatrixInformationWriteList,
)
from ._partial_bid_matrix_information_with_scenarios import (
    PartialBidMatrixInformationWithScenarios,
    PartialBidMatrixInformationWithScenariosApply,
    PartialBidMatrixInformationWithScenariosApplyList,
    PartialBidMatrixInformationWithScenariosFields,
    PartialBidMatrixInformationWithScenariosGraphQL,
    PartialBidMatrixInformationWithScenariosList,
    PartialBidMatrixInformationWithScenariosTextFields,
    PartialBidMatrixInformationWithScenariosWrite,
    PartialBidMatrixInformationWithScenariosWriteList,
)
from ._plant import (
    Plant,
    PlantApply,
    PlantApplyList,
    PlantFields,
    PlantGraphQL,
    PlantList,
    PlantTextFields,
    PlantWrite,
    PlantWriteList,
)
from ._plant_information import (
    PlantInformation,
    PlantInformationApply,
    PlantInformationApplyList,
    PlantInformationFields,
    PlantInformationGraphQL,
    PlantInformationList,
    PlantInformationTextFields,
    PlantInformationWrite,
    PlantInformationWriteList,
)
from ._plant_water_value_based import (
    PlantWaterValueBased,
    PlantWaterValueBasedApply,
    PlantWaterValueBasedApplyList,
    PlantWaterValueBasedFields,
    PlantWaterValueBasedGraphQL,
    PlantWaterValueBasedList,
    PlantWaterValueBasedTextFields,
    PlantWaterValueBasedWrite,
    PlantWaterValueBasedWriteList,
)
from ._power_asset import (
    PowerAsset,
    PowerAssetApply,
    PowerAssetApplyList,
    PowerAssetFields,
    PowerAssetGraphQL,
    PowerAssetList,
    PowerAssetTextFields,
    PowerAssetWrite,
    PowerAssetWriteList,
)
from ._price_area import (
    PriceArea,
    PriceAreaApply,
    PriceAreaApplyList,
    PriceAreaFields,
    PriceAreaGraphQL,
    PriceAreaList,
    PriceAreaTextFields,
    PriceAreaWrite,
    PriceAreaWriteList,
)
from ._price_area_afrr import (
    PriceAreaAFRR,
    PriceAreaAFRRApply,
    PriceAreaAFRRApplyList,
    PriceAreaAFRRFields,
    PriceAreaAFRRGraphQL,
    PriceAreaAFRRList,
    PriceAreaAFRRTextFields,
    PriceAreaAFRRWrite,
    PriceAreaAFRRWriteList,
)
from ._price_area_day_ahead import (
    PriceAreaDayAhead,
    PriceAreaDayAheadApply,
    PriceAreaDayAheadApplyList,
    PriceAreaDayAheadFields,
    PriceAreaDayAheadGraphQL,
    PriceAreaDayAheadList,
    PriceAreaDayAheadTextFields,
    PriceAreaDayAheadWrite,
    PriceAreaDayAheadWriteList,
)
from ._price_area_information import (
    PriceAreaInformation,
    PriceAreaInformationApply,
    PriceAreaInformationApplyList,
    PriceAreaInformationFields,
    PriceAreaInformationGraphQL,
    PriceAreaInformationList,
    PriceAreaInformationTextFields,
    PriceAreaInformationWrite,
    PriceAreaInformationWriteList,
)
from ._price_production import (
    PriceProduction,
    PriceProductionApply,
    PriceProductionApplyList,
    PriceProductionFields,
    PriceProductionGraphQL,
    PriceProductionList,
    PriceProductionTextFields,
    PriceProductionWrite,
    PriceProductionWriteList,
)
from ._shop_attribute_mapping import (
    ShopAttributeMapping,
    ShopAttributeMappingApply,
    ShopAttributeMappingApplyList,
    ShopAttributeMappingFields,
    ShopAttributeMappingGraphQL,
    ShopAttributeMappingList,
    ShopAttributeMappingTextFields,
    ShopAttributeMappingWrite,
    ShopAttributeMappingWriteList,
)
from ._shop_based_partial_bid_configuration import (
    ShopBasedPartialBidConfiguration,
    ShopBasedPartialBidConfigurationApply,
    ShopBasedPartialBidConfigurationApplyList,
    ShopBasedPartialBidConfigurationFields,
    ShopBasedPartialBidConfigurationGraphQL,
    ShopBasedPartialBidConfigurationList,
    ShopBasedPartialBidConfigurationTextFields,
    ShopBasedPartialBidConfigurationWrite,
    ShopBasedPartialBidConfigurationWriteList,
)
from ._shop_case import (
    ShopCase,
    ShopCaseApply,
    ShopCaseApplyList,
    ShopCaseFields,
    ShopCaseGraphQL,
    ShopCaseList,
    ShopCaseWrite,
    ShopCaseWriteList,
)
from ._shop_commands import (
    ShopCommands,
    ShopCommandsApply,
    ShopCommandsApplyList,
    ShopCommandsFields,
    ShopCommandsGraphQL,
    ShopCommandsList,
    ShopCommandsTextFields,
    ShopCommandsWrite,
    ShopCommandsWriteList,
)
from ._shop_file import (
    ShopFile,
    ShopFileApply,
    ShopFileApplyList,
    ShopFileFields,
    ShopFileGraphQL,
    ShopFileList,
    ShopFileTextFields,
    ShopFileWrite,
    ShopFileWriteList,
)
from ._shop_model import (
    ShopModel,
    ShopModelApply,
    ShopModelApplyList,
    ShopModelFields,
    ShopModelGraphQL,
    ShopModelList,
    ShopModelTextFields,
    ShopModelWrite,
    ShopModelWriteList,
)
from ._shop_model_with_assets import (
    ShopModelWithAssets,
    ShopModelWithAssetsApply,
    ShopModelWithAssetsApplyList,
    ShopModelWithAssetsGraphQL,
    ShopModelWithAssetsList,
    ShopModelWithAssetsWrite,
    ShopModelWithAssetsWriteList,
)
from ._shop_output_time_series_definition import (
    ShopOutputTimeSeriesDefinition,
    ShopOutputTimeSeriesDefinitionApply,
    ShopOutputTimeSeriesDefinitionApplyList,
    ShopOutputTimeSeriesDefinitionFields,
    ShopOutputTimeSeriesDefinitionGraphQL,
    ShopOutputTimeSeriesDefinitionList,
    ShopOutputTimeSeriesDefinitionTextFields,
    ShopOutputTimeSeriesDefinitionWrite,
    ShopOutputTimeSeriesDefinitionWriteList,
)
from ._shop_penalty_report import (
    ShopPenaltyReport,
    ShopPenaltyReportApply,
    ShopPenaltyReportApplyList,
    ShopPenaltyReportFields,
    ShopPenaltyReportGraphQL,
    ShopPenaltyReportList,
    ShopPenaltyReportTextFields,
    ShopPenaltyReportWrite,
    ShopPenaltyReportWriteList,
)
from ._shop_preprocessor_input import (
    ShopPreprocessorInput,
    ShopPreprocessorInputApply,
    ShopPreprocessorInputApplyList,
    ShopPreprocessorInputFields,
    ShopPreprocessorInputGraphQL,
    ShopPreprocessorInputList,
    ShopPreprocessorInputTextFields,
    ShopPreprocessorInputWrite,
    ShopPreprocessorInputWriteList,
)
from ._shop_preprocessor_output import (
    ShopPreprocessorOutput,
    ShopPreprocessorOutputApply,
    ShopPreprocessorOutputApplyList,
    ShopPreprocessorOutputFields,
    ShopPreprocessorOutputGraphQL,
    ShopPreprocessorOutputList,
    ShopPreprocessorOutputTextFields,
    ShopPreprocessorOutputWrite,
    ShopPreprocessorOutputWriteList,
)
from ._shop_result import (
    ShopResult,
    ShopResultApply,
    ShopResultApplyList,
    ShopResultFields,
    ShopResultGraphQL,
    ShopResultList,
    ShopResultWrite,
    ShopResultWriteList,
)
from ._shop_scenario import (
    ShopScenario,
    ShopScenarioApply,
    ShopScenarioApplyList,
    ShopScenarioFields,
    ShopScenarioGraphQL,
    ShopScenarioList,
    ShopScenarioTextFields,
    ShopScenarioWrite,
    ShopScenarioWriteList,
)
from ._shop_scenario_set import (
    ShopScenarioSet,
    ShopScenarioSetApply,
    ShopScenarioSetApplyList,
    ShopScenarioSetFields,
    ShopScenarioSetGraphQL,
    ShopScenarioSetList,
    ShopScenarioSetTextFields,
    ShopScenarioSetWrite,
    ShopScenarioSetWriteList,
)
from ._shop_time_series import (
    ShopTimeSeries,
    ShopTimeSeriesApply,
    ShopTimeSeriesApplyList,
    ShopTimeSeriesFields,
    ShopTimeSeriesGraphQL,
    ShopTimeSeriesList,
    ShopTimeSeriesTextFields,
    ShopTimeSeriesWrite,
    ShopTimeSeriesWriteList,
)
from ._shop_trigger_input import (
    ShopTriggerInput,
    ShopTriggerInputApply,
    ShopTriggerInputApplyList,
    ShopTriggerInputFields,
    ShopTriggerInputGraphQL,
    ShopTriggerInputList,
    ShopTriggerInputTextFields,
    ShopTriggerInputWrite,
    ShopTriggerInputWriteList,
)
from ._shop_trigger_output import (
    ShopTriggerOutput,
    ShopTriggerOutputApply,
    ShopTriggerOutputApplyList,
    ShopTriggerOutputFields,
    ShopTriggerOutputGraphQL,
    ShopTriggerOutputList,
    ShopTriggerOutputTextFields,
    ShopTriggerOutputWrite,
    ShopTriggerOutputWriteList,
)
from ._task_dispatcher_input import (
    TaskDispatcherInput,
    TaskDispatcherInputApply,
    TaskDispatcherInputApplyList,
    TaskDispatcherInputFields,
    TaskDispatcherInputGraphQL,
    TaskDispatcherInputList,
    TaskDispatcherInputTextFields,
    TaskDispatcherInputWrite,
    TaskDispatcherInputWriteList,
)
from ._task_dispatcher_output import (
    TaskDispatcherOutput,
    TaskDispatcherOutputApply,
    TaskDispatcherOutputApplyList,
    TaskDispatcherOutputFields,
    TaskDispatcherOutputGraphQL,
    TaskDispatcherOutputList,
    TaskDispatcherOutputTextFields,
    TaskDispatcherOutputWrite,
    TaskDispatcherOutputWriteList,
)
from ._total_bid_matrix_calculation_input import (
    TotalBidMatrixCalculationInput,
    TotalBidMatrixCalculationInputApply,
    TotalBidMatrixCalculationInputApplyList,
    TotalBidMatrixCalculationInputFields,
    TotalBidMatrixCalculationInputGraphQL,
    TotalBidMatrixCalculationInputList,
    TotalBidMatrixCalculationInputTextFields,
    TotalBidMatrixCalculationInputWrite,
    TotalBidMatrixCalculationInputWriteList,
)
from ._total_bid_matrix_calculation_output import (
    TotalBidMatrixCalculationOutput,
    TotalBidMatrixCalculationOutputApply,
    TotalBidMatrixCalculationOutputApplyList,
    TotalBidMatrixCalculationOutputFields,
    TotalBidMatrixCalculationOutputGraphQL,
    TotalBidMatrixCalculationOutputList,
    TotalBidMatrixCalculationOutputTextFields,
    TotalBidMatrixCalculationOutputWrite,
    TotalBidMatrixCalculationOutputWriteList,
)
from ._turbine_efficiency_curve import (
    TurbineEfficiencyCurve,
    TurbineEfficiencyCurveApply,
    TurbineEfficiencyCurveApplyList,
    TurbineEfficiencyCurveFields,
    TurbineEfficiencyCurveGraphQL,
    TurbineEfficiencyCurveList,
    TurbineEfficiencyCurveWrite,
    TurbineEfficiencyCurveWriteList,
)
from ._water_value_based_partial_bid_configuration import (
    WaterValueBasedPartialBidConfiguration,
    WaterValueBasedPartialBidConfigurationApply,
    WaterValueBasedPartialBidConfigurationApplyList,
    WaterValueBasedPartialBidConfigurationFields,
    WaterValueBasedPartialBidConfigurationGraphQL,
    WaterValueBasedPartialBidConfigurationList,
    WaterValueBasedPartialBidConfigurationTextFields,
    WaterValueBasedPartialBidConfigurationWrite,
    WaterValueBasedPartialBidConfigurationWriteList,
)
from ._water_value_based_partial_bid_matrix_calculation_input import (
    WaterValueBasedPartialBidMatrixCalculationInput,
    WaterValueBasedPartialBidMatrixCalculationInputApply,
    WaterValueBasedPartialBidMatrixCalculationInputApplyList,
    WaterValueBasedPartialBidMatrixCalculationInputFields,
    WaterValueBasedPartialBidMatrixCalculationInputGraphQL,
    WaterValueBasedPartialBidMatrixCalculationInputList,
    WaterValueBasedPartialBidMatrixCalculationInputTextFields,
    WaterValueBasedPartialBidMatrixCalculationInputWrite,
    WaterValueBasedPartialBidMatrixCalculationInputWriteList,
)
from ._watercourse import (
    Watercourse,
    WatercourseApply,
    WatercourseApplyList,
    WatercourseFields,
    WatercourseGraphQL,
    WatercourseList,
    WatercourseTextFields,
    WatercourseWrite,
    WatercourseWriteList,
)

BenchmarkingCalculationInput.model_rebuild()
BenchmarkingCalculationInputGraphQL.model_rebuild()
BenchmarkingCalculationInputWrite.model_rebuild()
BenchmarkingCalculationInputApply.model_rebuild()
BenchmarkingCalculationOutput.model_rebuild()
BenchmarkingCalculationOutputGraphQL.model_rebuild()
BenchmarkingCalculationOutputWrite.model_rebuild()
BenchmarkingCalculationOutputApply.model_rebuild()
BenchmarkingConfigurationDayAhead.model_rebuild()
BenchmarkingConfigurationDayAheadGraphQL.model_rebuild()
BenchmarkingConfigurationDayAheadWrite.model_rebuild()
BenchmarkingConfigurationDayAheadApply.model_rebuild()
BenchmarkingResultDayAhead.model_rebuild()
BenchmarkingResultDayAheadGraphQL.model_rebuild()
BenchmarkingResultDayAheadWrite.model_rebuild()
BenchmarkingResultDayAheadApply.model_rebuild()
BenchmarkingShopCase.model_rebuild()
BenchmarkingShopCaseGraphQL.model_rebuild()
BenchmarkingShopCaseWrite.model_rebuild()
BenchmarkingShopCaseApply.model_rebuild()
BenchmarkingTaskDispatcherInputDayAhead.model_rebuild()
BenchmarkingTaskDispatcherInputDayAheadGraphQL.model_rebuild()
BenchmarkingTaskDispatcherInputDayAheadWrite.model_rebuild()
BenchmarkingTaskDispatcherInputDayAheadApply.model_rebuild()
BenchmarkingTaskDispatcherOutputDayAhead.model_rebuild()
BenchmarkingTaskDispatcherOutputDayAheadGraphQL.model_rebuild()
BenchmarkingTaskDispatcherOutputDayAheadWrite.model_rebuild()
BenchmarkingTaskDispatcherOutputDayAheadApply.model_rebuild()
BidConfigurationDayAhead.model_rebuild()
BidConfigurationDayAheadGraphQL.model_rebuild()
BidConfigurationDayAheadWrite.model_rebuild()
BidConfigurationDayAheadApply.model_rebuild()
BidDocument.model_rebuild()
BidDocumentGraphQL.model_rebuild()
BidDocumentWrite.model_rebuild()
BidDocumentApply.model_rebuild()
BidDocumentAFRR.model_rebuild()
BidDocumentAFRRGraphQL.model_rebuild()
BidDocumentAFRRWrite.model_rebuild()
BidDocumentAFRRApply.model_rebuild()
BidDocumentDayAhead.model_rebuild()
BidDocumentDayAheadGraphQL.model_rebuild()
BidDocumentDayAheadWrite.model_rebuild()
BidDocumentDayAheadApply.model_rebuild()
BidMatrixInformation.model_rebuild()
BidMatrixInformationGraphQL.model_rebuild()
BidMatrixInformationWrite.model_rebuild()
BidMatrixInformationApply.model_rebuild()
BidRow.model_rebuild()
BidRowGraphQL.model_rebuild()
BidRowWrite.model_rebuild()
BidRowApply.model_rebuild()
FunctionOutput.model_rebuild()
FunctionOutputGraphQL.model_rebuild()
FunctionOutputWrite.model_rebuild()
FunctionOutputApply.model_rebuild()
Generator.model_rebuild()
GeneratorGraphQL.model_rebuild()
GeneratorWrite.model_rebuild()
GeneratorApply.model_rebuild()
MultiScenarioPartialBidMatrixCalculationInput.model_rebuild()
MultiScenarioPartialBidMatrixCalculationInputGraphQL.model_rebuild()
MultiScenarioPartialBidMatrixCalculationInputWrite.model_rebuild()
MultiScenarioPartialBidMatrixCalculationInputApply.model_rebuild()
PartialBidConfiguration.model_rebuild()
PartialBidConfigurationGraphQL.model_rebuild()
PartialBidConfigurationWrite.model_rebuild()
PartialBidConfigurationApply.model_rebuild()
PartialBidMatrixCalculationInput.model_rebuild()
PartialBidMatrixCalculationInputGraphQL.model_rebuild()
PartialBidMatrixCalculationInputWrite.model_rebuild()
PartialBidMatrixCalculationInputApply.model_rebuild()
PartialBidMatrixCalculationOutput.model_rebuild()
PartialBidMatrixCalculationOutputGraphQL.model_rebuild()
PartialBidMatrixCalculationOutputWrite.model_rebuild()
PartialBidMatrixCalculationOutputApply.model_rebuild()
PartialBidMatrixInformation.model_rebuild()
PartialBidMatrixInformationGraphQL.model_rebuild()
PartialBidMatrixInformationWrite.model_rebuild()
PartialBidMatrixInformationApply.model_rebuild()
PartialBidMatrixInformationWithScenarios.model_rebuild()
PartialBidMatrixInformationWithScenariosGraphQL.model_rebuild()
PartialBidMatrixInformationWithScenariosWrite.model_rebuild()
PartialBidMatrixInformationWithScenariosApply.model_rebuild()
PlantInformation.model_rebuild()
PlantInformationGraphQL.model_rebuild()
PlantInformationWrite.model_rebuild()
PlantInformationApply.model_rebuild()
PlantWaterValueBased.model_rebuild()
PlantWaterValueBasedGraphQL.model_rebuild()
PlantWaterValueBasedWrite.model_rebuild()
PlantWaterValueBasedApply.model_rebuild()
PriceAreaDayAhead.model_rebuild()
PriceAreaDayAheadGraphQL.model_rebuild()
PriceAreaDayAheadWrite.model_rebuild()
PriceAreaDayAheadApply.model_rebuild()
PriceAreaInformation.model_rebuild()
PriceAreaInformationGraphQL.model_rebuild()
PriceAreaInformationWrite.model_rebuild()
PriceAreaInformationApply.model_rebuild()
PriceProduction.model_rebuild()
PriceProductionGraphQL.model_rebuild()
PriceProductionWrite.model_rebuild()
PriceProductionApply.model_rebuild()
ShopBasedPartialBidConfiguration.model_rebuild()
ShopBasedPartialBidConfigurationGraphQL.model_rebuild()
ShopBasedPartialBidConfigurationWrite.model_rebuild()
ShopBasedPartialBidConfigurationApply.model_rebuild()
ShopCase.model_rebuild()
ShopCaseGraphQL.model_rebuild()
ShopCaseWrite.model_rebuild()
ShopCaseApply.model_rebuild()
ShopModel.model_rebuild()
ShopModelGraphQL.model_rebuild()
ShopModelWrite.model_rebuild()
ShopModelApply.model_rebuild()
ShopModelWithAssets.model_rebuild()
ShopModelWithAssetsGraphQL.model_rebuild()
ShopModelWithAssetsWrite.model_rebuild()
ShopModelWithAssetsApply.model_rebuild()
ShopPreprocessorInput.model_rebuild()
ShopPreprocessorInputGraphQL.model_rebuild()
ShopPreprocessorInputWrite.model_rebuild()
ShopPreprocessorInputApply.model_rebuild()
ShopPreprocessorOutput.model_rebuild()
ShopPreprocessorOutputGraphQL.model_rebuild()
ShopPreprocessorOutputWrite.model_rebuild()
ShopPreprocessorOutputApply.model_rebuild()
ShopResult.model_rebuild()
ShopResultGraphQL.model_rebuild()
ShopResultWrite.model_rebuild()
ShopResultApply.model_rebuild()
ShopScenario.model_rebuild()
ShopScenarioGraphQL.model_rebuild()
ShopScenarioWrite.model_rebuild()
ShopScenarioApply.model_rebuild()
ShopScenarioSet.model_rebuild()
ShopScenarioSetGraphQL.model_rebuild()
ShopScenarioSetWrite.model_rebuild()
ShopScenarioSetApply.model_rebuild()
ShopTriggerInput.model_rebuild()
ShopTriggerInputGraphQL.model_rebuild()
ShopTriggerInputWrite.model_rebuild()
ShopTriggerInputApply.model_rebuild()
ShopTriggerOutput.model_rebuild()
ShopTriggerOutputGraphQL.model_rebuild()
ShopTriggerOutputWrite.model_rebuild()
ShopTriggerOutputApply.model_rebuild()
TaskDispatcherInput.model_rebuild()
TaskDispatcherInputGraphQL.model_rebuild()
TaskDispatcherInputWrite.model_rebuild()
TaskDispatcherInputApply.model_rebuild()
TaskDispatcherOutput.model_rebuild()
TaskDispatcherOutputGraphQL.model_rebuild()
TaskDispatcherOutputWrite.model_rebuild()
TaskDispatcherOutputApply.model_rebuild()
TotalBidMatrixCalculationInput.model_rebuild()
TotalBidMatrixCalculationInputGraphQL.model_rebuild()
TotalBidMatrixCalculationInputWrite.model_rebuild()
TotalBidMatrixCalculationInputApply.model_rebuild()
TotalBidMatrixCalculationOutput.model_rebuild()
TotalBidMatrixCalculationOutputGraphQL.model_rebuild()
TotalBidMatrixCalculationOutputWrite.model_rebuild()
TotalBidMatrixCalculationOutputApply.model_rebuild()
WaterValueBasedPartialBidConfiguration.model_rebuild()
WaterValueBasedPartialBidConfigurationGraphQL.model_rebuild()
WaterValueBasedPartialBidConfigurationWrite.model_rebuild()
WaterValueBasedPartialBidConfigurationApply.model_rebuild()
WaterValueBasedPartialBidMatrixCalculationInput.model_rebuild()
WaterValueBasedPartialBidMatrixCalculationInputGraphQL.model_rebuild()
WaterValueBasedPartialBidMatrixCalculationInputWrite.model_rebuild()
WaterValueBasedPartialBidMatrixCalculationInputApply.model_rebuild()


__all__ = [
    "DataRecord",
    "DataRecordGraphQL",
    "DataRecordWrite",
    "ResourcesWrite",
    "DomainModel",
    "DomainModelCore",
    "DomainModelWrite",
    "DomainModelList",
    "DomainRelationWrite",
    "GraphQLCore",
    "GraphQLList",
    "ResourcesWriteResult",
    "PageInfo",
    "Alert",
    "AlertGraphQL",
    "AlertWrite",
    "AlertApply",
    "AlertList",
    "AlertWriteList",
    "AlertApplyList",
    "AlertFields",
    "AlertTextFields",
    "BenchmarkingCalculationInput",
    "BenchmarkingCalculationInputGraphQL",
    "BenchmarkingCalculationInputWrite",
    "BenchmarkingCalculationInputApply",
    "BenchmarkingCalculationInputList",
    "BenchmarkingCalculationInputWriteList",
    "BenchmarkingCalculationInputApplyList",
    "BenchmarkingCalculationInputFields",
    "BenchmarkingCalculationInputTextFields",
    "BenchmarkingCalculationOutput",
    "BenchmarkingCalculationOutputGraphQL",
    "BenchmarkingCalculationOutputWrite",
    "BenchmarkingCalculationOutputApply",
    "BenchmarkingCalculationOutputList",
    "BenchmarkingCalculationOutputWriteList",
    "BenchmarkingCalculationOutputApplyList",
    "BenchmarkingCalculationOutputFields",
    "BenchmarkingCalculationOutputTextFields",
    "BenchmarkingConfigurationDayAhead",
    "BenchmarkingConfigurationDayAheadGraphQL",
    "BenchmarkingConfigurationDayAheadWrite",
    "BenchmarkingConfigurationDayAheadApply",
    "BenchmarkingConfigurationDayAheadList",
    "BenchmarkingConfigurationDayAheadWriteList",
    "BenchmarkingConfigurationDayAheadApplyList",
    "BenchmarkingConfigurationDayAheadFields",
    "BenchmarkingConfigurationDayAheadTextFields",
    "BenchmarkingProductionObligationDayAhead",
    "BenchmarkingProductionObligationDayAheadGraphQL",
    "BenchmarkingProductionObligationDayAheadWrite",
    "BenchmarkingProductionObligationDayAheadApply",
    "BenchmarkingProductionObligationDayAheadList",
    "BenchmarkingProductionObligationDayAheadWriteList",
    "BenchmarkingProductionObligationDayAheadApplyList",
    "BenchmarkingProductionObligationDayAheadFields",
    "BenchmarkingProductionObligationDayAheadTextFields",
    "BenchmarkingResultDayAhead",
    "BenchmarkingResultDayAheadGraphQL",
    "BenchmarkingResultDayAheadWrite",
    "BenchmarkingResultDayAheadApply",
    "BenchmarkingResultDayAheadList",
    "BenchmarkingResultDayAheadWriteList",
    "BenchmarkingResultDayAheadApplyList",
    "BenchmarkingResultDayAheadFields",
    "BenchmarkingResultDayAheadTextFields",
    "BenchmarkingShopCase",
    "BenchmarkingShopCaseGraphQL",
    "BenchmarkingShopCaseWrite",
    "BenchmarkingShopCaseApply",
    "BenchmarkingShopCaseList",
    "BenchmarkingShopCaseWriteList",
    "BenchmarkingShopCaseApplyList",
    "BenchmarkingShopCaseFields",
    "BenchmarkingTaskDispatcherInputDayAhead",
    "BenchmarkingTaskDispatcherInputDayAheadGraphQL",
    "BenchmarkingTaskDispatcherInputDayAheadWrite",
    "BenchmarkingTaskDispatcherInputDayAheadApply",
    "BenchmarkingTaskDispatcherInputDayAheadList",
    "BenchmarkingTaskDispatcherInputDayAheadWriteList",
    "BenchmarkingTaskDispatcherInputDayAheadApplyList",
    "BenchmarkingTaskDispatcherInputDayAheadFields",
    "BenchmarkingTaskDispatcherInputDayAheadTextFields",
    "BenchmarkingTaskDispatcherOutputDayAhead",
    "BenchmarkingTaskDispatcherOutputDayAheadGraphQL",
    "BenchmarkingTaskDispatcherOutputDayAheadWrite",
    "BenchmarkingTaskDispatcherOutputDayAheadApply",
    "BenchmarkingTaskDispatcherOutputDayAheadList",
    "BenchmarkingTaskDispatcherOutputDayAheadWriteList",
    "BenchmarkingTaskDispatcherOutputDayAheadApplyList",
    "BenchmarkingTaskDispatcherOutputDayAheadFields",
    "BenchmarkingTaskDispatcherOutputDayAheadTextFields",
    "BidConfigurationDayAhead",
    "BidConfigurationDayAheadGraphQL",
    "BidConfigurationDayAheadWrite",
    "BidConfigurationDayAheadApply",
    "BidConfigurationDayAheadList",
    "BidConfigurationDayAheadWriteList",
    "BidConfigurationDayAheadApplyList",
    "BidConfigurationDayAheadFields",
    "BidConfigurationDayAheadTextFields",
    "BidDocument",
    "BidDocumentGraphQL",
    "BidDocumentWrite",
    "BidDocumentApply",
    "BidDocumentList",
    "BidDocumentWriteList",
    "BidDocumentApplyList",
    "BidDocumentFields",
    "BidDocumentTextFields",
    "BidDocumentAFRR",
    "BidDocumentAFRRGraphQL",
    "BidDocumentAFRRWrite",
    "BidDocumentAFRRApply",
    "BidDocumentAFRRList",
    "BidDocumentAFRRWriteList",
    "BidDocumentAFRRApplyList",
    "BidDocumentAFRRFields",
    "BidDocumentAFRRTextFields",
    "BidDocumentDayAhead",
    "BidDocumentDayAheadGraphQL",
    "BidDocumentDayAheadWrite",
    "BidDocumentDayAheadApply",
    "BidDocumentDayAheadList",
    "BidDocumentDayAheadWriteList",
    "BidDocumentDayAheadApplyList",
    "BidDocumentDayAheadFields",
    "BidDocumentDayAheadTextFields",
    "BidMatrix",
    "BidMatrixGraphQL",
    "BidMatrixWrite",
    "BidMatrixApply",
    "BidMatrixList",
    "BidMatrixWriteList",
    "BidMatrixApplyList",
    "BidMatrixFields",
    "BidMatrixTextFields",
    "BidMatrixInformation",
    "BidMatrixInformationGraphQL",
    "BidMatrixInformationWrite",
    "BidMatrixInformationApply",
    "BidMatrixInformationList",
    "BidMatrixInformationWriteList",
    "BidMatrixInformationApplyList",
    "BidMatrixInformationFields",
    "BidMatrixInformationTextFields",
    "BidRow",
    "BidRowGraphQL",
    "BidRowWrite",
    "BidRowApply",
    "BidRowList",
    "BidRowWriteList",
    "BidRowApplyList",
    "BidRowFields",
    "BidRowTextFields",
    "DateSpecification",
    "DateSpecificationGraphQL",
    "DateSpecificationWrite",
    "DateSpecificationApply",
    "DateSpecificationList",
    "DateSpecificationWriteList",
    "DateSpecificationApplyList",
    "DateSpecificationFields",
    "DateSpecificationTextFields",
    "FunctionInput",
    "FunctionInputGraphQL",
    "FunctionInputWrite",
    "FunctionInputApply",
    "FunctionInputList",
    "FunctionInputWriteList",
    "FunctionInputApplyList",
    "FunctionInputFields",
    "FunctionInputTextFields",
    "FunctionOutput",
    "FunctionOutputGraphQL",
    "FunctionOutputWrite",
    "FunctionOutputApply",
    "FunctionOutputList",
    "FunctionOutputWriteList",
    "FunctionOutputApplyList",
    "FunctionOutputFields",
    "FunctionOutputTextFields",
    "Generator",
    "GeneratorGraphQL",
    "GeneratorWrite",
    "GeneratorApply",
    "GeneratorList",
    "GeneratorWriteList",
    "GeneratorApplyList",
    "GeneratorFields",
    "GeneratorTextFields",
    "GeneratorEfficiencyCurve",
    "GeneratorEfficiencyCurveGraphQL",
    "GeneratorEfficiencyCurveWrite",
    "GeneratorEfficiencyCurveApply",
    "GeneratorEfficiencyCurveList",
    "GeneratorEfficiencyCurveWriteList",
    "GeneratorEfficiencyCurveApplyList",
    "GeneratorEfficiencyCurveFields",
    "MarketConfiguration",
    "MarketConfigurationGraphQL",
    "MarketConfigurationWrite",
    "MarketConfigurationApply",
    "MarketConfigurationList",
    "MarketConfigurationWriteList",
    "MarketConfigurationApplyList",
    "MarketConfigurationFields",
    "MarketConfigurationTextFields",
    "MultiScenarioPartialBidMatrixCalculationInput",
    "MultiScenarioPartialBidMatrixCalculationInputGraphQL",
    "MultiScenarioPartialBidMatrixCalculationInputWrite",
    "MultiScenarioPartialBidMatrixCalculationInputApply",
    "MultiScenarioPartialBidMatrixCalculationInputList",
    "MultiScenarioPartialBidMatrixCalculationInputWriteList",
    "MultiScenarioPartialBidMatrixCalculationInputApplyList",
    "MultiScenarioPartialBidMatrixCalculationInputFields",
    "MultiScenarioPartialBidMatrixCalculationInputTextFields",
    "PartialBidConfiguration",
    "PartialBidConfigurationGraphQL",
    "PartialBidConfigurationWrite",
    "PartialBidConfigurationApply",
    "PartialBidConfigurationList",
    "PartialBidConfigurationWriteList",
    "PartialBidConfigurationApplyList",
    "PartialBidConfigurationFields",
    "PartialBidConfigurationTextFields",
    "PartialBidMatrixCalculationInput",
    "PartialBidMatrixCalculationInputGraphQL",
    "PartialBidMatrixCalculationInputWrite",
    "PartialBidMatrixCalculationInputApply",
    "PartialBidMatrixCalculationInputList",
    "PartialBidMatrixCalculationInputWriteList",
    "PartialBidMatrixCalculationInputApplyList",
    "PartialBidMatrixCalculationInputFields",
    "PartialBidMatrixCalculationInputTextFields",
    "PartialBidMatrixCalculationOutput",
    "PartialBidMatrixCalculationOutputGraphQL",
    "PartialBidMatrixCalculationOutputWrite",
    "PartialBidMatrixCalculationOutputApply",
    "PartialBidMatrixCalculationOutputList",
    "PartialBidMatrixCalculationOutputWriteList",
    "PartialBidMatrixCalculationOutputApplyList",
    "PartialBidMatrixCalculationOutputFields",
    "PartialBidMatrixCalculationOutputTextFields",
    "PartialBidMatrixInformation",
    "PartialBidMatrixInformationGraphQL",
    "PartialBidMatrixInformationWrite",
    "PartialBidMatrixInformationApply",
    "PartialBidMatrixInformationList",
    "PartialBidMatrixInformationWriteList",
    "PartialBidMatrixInformationApplyList",
    "PartialBidMatrixInformationFields",
    "PartialBidMatrixInformationTextFields",
    "PartialBidMatrixInformationWithScenarios",
    "PartialBidMatrixInformationWithScenariosGraphQL",
    "PartialBidMatrixInformationWithScenariosWrite",
    "PartialBidMatrixInformationWithScenariosApply",
    "PartialBidMatrixInformationWithScenariosList",
    "PartialBidMatrixInformationWithScenariosWriteList",
    "PartialBidMatrixInformationWithScenariosApplyList",
    "PartialBidMatrixInformationWithScenariosFields",
    "PartialBidMatrixInformationWithScenariosTextFields",
    "Plant",
    "PlantGraphQL",
    "PlantWrite",
    "PlantApply",
    "PlantList",
    "PlantWriteList",
    "PlantApplyList",
    "PlantFields",
    "PlantTextFields",
    "PlantInformation",
    "PlantInformationGraphQL",
    "PlantInformationWrite",
    "PlantInformationApply",
    "PlantInformationList",
    "PlantInformationWriteList",
    "PlantInformationApplyList",
    "PlantInformationFields",
    "PlantInformationTextFields",
    "PlantWaterValueBased",
    "PlantWaterValueBasedGraphQL",
    "PlantWaterValueBasedWrite",
    "PlantWaterValueBasedApply",
    "PlantWaterValueBasedList",
    "PlantWaterValueBasedWriteList",
    "PlantWaterValueBasedApplyList",
    "PlantWaterValueBasedFields",
    "PlantWaterValueBasedTextFields",
    "PowerAsset",
    "PowerAssetGraphQL",
    "PowerAssetWrite",
    "PowerAssetApply",
    "PowerAssetList",
    "PowerAssetWriteList",
    "PowerAssetApplyList",
    "PowerAssetFields",
    "PowerAssetTextFields",
    "PriceArea",
    "PriceAreaGraphQL",
    "PriceAreaWrite",
    "PriceAreaApply",
    "PriceAreaList",
    "PriceAreaWriteList",
    "PriceAreaApplyList",
    "PriceAreaFields",
    "PriceAreaTextFields",
    "PriceAreaAFRR",
    "PriceAreaAFRRGraphQL",
    "PriceAreaAFRRWrite",
    "PriceAreaAFRRApply",
    "PriceAreaAFRRList",
    "PriceAreaAFRRWriteList",
    "PriceAreaAFRRApplyList",
    "PriceAreaAFRRFields",
    "PriceAreaAFRRTextFields",
    "PriceAreaDayAhead",
    "PriceAreaDayAheadGraphQL",
    "PriceAreaDayAheadWrite",
    "PriceAreaDayAheadApply",
    "PriceAreaDayAheadList",
    "PriceAreaDayAheadWriteList",
    "PriceAreaDayAheadApplyList",
    "PriceAreaDayAheadFields",
    "PriceAreaDayAheadTextFields",
    "PriceAreaInformation",
    "PriceAreaInformationGraphQL",
    "PriceAreaInformationWrite",
    "PriceAreaInformationApply",
    "PriceAreaInformationList",
    "PriceAreaInformationWriteList",
    "PriceAreaInformationApplyList",
    "PriceAreaInformationFields",
    "PriceAreaInformationTextFields",
    "PriceProduction",
    "PriceProductionGraphQL",
    "PriceProductionWrite",
    "PriceProductionApply",
    "PriceProductionList",
    "PriceProductionWriteList",
    "PriceProductionApplyList",
    "PriceProductionFields",
    "PriceProductionTextFields",
    "ShopAttributeMapping",
    "ShopAttributeMappingGraphQL",
    "ShopAttributeMappingWrite",
    "ShopAttributeMappingApply",
    "ShopAttributeMappingList",
    "ShopAttributeMappingWriteList",
    "ShopAttributeMappingApplyList",
    "ShopAttributeMappingFields",
    "ShopAttributeMappingTextFields",
    "ShopBasedPartialBidConfiguration",
    "ShopBasedPartialBidConfigurationGraphQL",
    "ShopBasedPartialBidConfigurationWrite",
    "ShopBasedPartialBidConfigurationApply",
    "ShopBasedPartialBidConfigurationList",
    "ShopBasedPartialBidConfigurationWriteList",
    "ShopBasedPartialBidConfigurationApplyList",
    "ShopBasedPartialBidConfigurationFields",
    "ShopBasedPartialBidConfigurationTextFields",
    "ShopCase",
    "ShopCaseGraphQL",
    "ShopCaseWrite",
    "ShopCaseApply",
    "ShopCaseList",
    "ShopCaseWriteList",
    "ShopCaseApplyList",
    "ShopCaseFields",
    "ShopCommands",
    "ShopCommandsGraphQL",
    "ShopCommandsWrite",
    "ShopCommandsApply",
    "ShopCommandsList",
    "ShopCommandsWriteList",
    "ShopCommandsApplyList",
    "ShopCommandsFields",
    "ShopCommandsTextFields",
    "ShopFile",
    "ShopFileGraphQL",
    "ShopFileWrite",
    "ShopFileApply",
    "ShopFileList",
    "ShopFileWriteList",
    "ShopFileApplyList",
    "ShopFileFields",
    "ShopFileTextFields",
    "ShopModel",
    "ShopModelGraphQL",
    "ShopModelWrite",
    "ShopModelApply",
    "ShopModelList",
    "ShopModelWriteList",
    "ShopModelApplyList",
    "ShopModelFields",
    "ShopModelTextFields",
    "ShopModelWithAssets",
    "ShopModelWithAssetsGraphQL",
    "ShopModelWithAssetsWrite",
    "ShopModelWithAssetsApply",
    "ShopModelWithAssetsList",
    "ShopModelWithAssetsWriteList",
    "ShopModelWithAssetsApplyList",
    "ShopOutputTimeSeriesDefinition",
    "ShopOutputTimeSeriesDefinitionGraphQL",
    "ShopOutputTimeSeriesDefinitionWrite",
    "ShopOutputTimeSeriesDefinitionApply",
    "ShopOutputTimeSeriesDefinitionList",
    "ShopOutputTimeSeriesDefinitionWriteList",
    "ShopOutputTimeSeriesDefinitionApplyList",
    "ShopOutputTimeSeriesDefinitionFields",
    "ShopOutputTimeSeriesDefinitionTextFields",
    "ShopPenaltyReport",
    "ShopPenaltyReportGraphQL",
    "ShopPenaltyReportWrite",
    "ShopPenaltyReportApply",
    "ShopPenaltyReportList",
    "ShopPenaltyReportWriteList",
    "ShopPenaltyReportApplyList",
    "ShopPenaltyReportFields",
    "ShopPenaltyReportTextFields",
    "ShopPreprocessorInput",
    "ShopPreprocessorInputGraphQL",
    "ShopPreprocessorInputWrite",
    "ShopPreprocessorInputApply",
    "ShopPreprocessorInputList",
    "ShopPreprocessorInputWriteList",
    "ShopPreprocessorInputApplyList",
    "ShopPreprocessorInputFields",
    "ShopPreprocessorInputTextFields",
    "ShopPreprocessorOutput",
    "ShopPreprocessorOutputGraphQL",
    "ShopPreprocessorOutputWrite",
    "ShopPreprocessorOutputApply",
    "ShopPreprocessorOutputList",
    "ShopPreprocessorOutputWriteList",
    "ShopPreprocessorOutputApplyList",
    "ShopPreprocessorOutputFields",
    "ShopPreprocessorOutputTextFields",
    "ShopResult",
    "ShopResultGraphQL",
    "ShopResultWrite",
    "ShopResultApply",
    "ShopResultList",
    "ShopResultWriteList",
    "ShopResultApplyList",
    "ShopResultFields",
    "ShopScenario",
    "ShopScenarioGraphQL",
    "ShopScenarioWrite",
    "ShopScenarioApply",
    "ShopScenarioList",
    "ShopScenarioWriteList",
    "ShopScenarioApplyList",
    "ShopScenarioFields",
    "ShopScenarioTextFields",
    "ShopScenarioSet",
    "ShopScenarioSetGraphQL",
    "ShopScenarioSetWrite",
    "ShopScenarioSetApply",
    "ShopScenarioSetList",
    "ShopScenarioSetWriteList",
    "ShopScenarioSetApplyList",
    "ShopScenarioSetFields",
    "ShopScenarioSetTextFields",
    "ShopTimeSeries",
    "ShopTimeSeriesGraphQL",
    "ShopTimeSeriesWrite",
    "ShopTimeSeriesApply",
    "ShopTimeSeriesList",
    "ShopTimeSeriesWriteList",
    "ShopTimeSeriesApplyList",
    "ShopTimeSeriesFields",
    "ShopTimeSeriesTextFields",
    "ShopTriggerInput",
    "ShopTriggerInputGraphQL",
    "ShopTriggerInputWrite",
    "ShopTriggerInputApply",
    "ShopTriggerInputList",
    "ShopTriggerInputWriteList",
    "ShopTriggerInputApplyList",
    "ShopTriggerInputFields",
    "ShopTriggerInputTextFields",
    "ShopTriggerOutput",
    "ShopTriggerOutputGraphQL",
    "ShopTriggerOutputWrite",
    "ShopTriggerOutputApply",
    "ShopTriggerOutputList",
    "ShopTriggerOutputWriteList",
    "ShopTriggerOutputApplyList",
    "ShopTriggerOutputFields",
    "ShopTriggerOutputTextFields",
    "TaskDispatcherInput",
    "TaskDispatcherInputGraphQL",
    "TaskDispatcherInputWrite",
    "TaskDispatcherInputApply",
    "TaskDispatcherInputList",
    "TaskDispatcherInputWriteList",
    "TaskDispatcherInputApplyList",
    "TaskDispatcherInputFields",
    "TaskDispatcherInputTextFields",
    "TaskDispatcherOutput",
    "TaskDispatcherOutputGraphQL",
    "TaskDispatcherOutputWrite",
    "TaskDispatcherOutputApply",
    "TaskDispatcherOutputList",
    "TaskDispatcherOutputWriteList",
    "TaskDispatcherOutputApplyList",
    "TaskDispatcherOutputFields",
    "TaskDispatcherOutputTextFields",
    "TotalBidMatrixCalculationInput",
    "TotalBidMatrixCalculationInputGraphQL",
    "TotalBidMatrixCalculationInputWrite",
    "TotalBidMatrixCalculationInputApply",
    "TotalBidMatrixCalculationInputList",
    "TotalBidMatrixCalculationInputWriteList",
    "TotalBidMatrixCalculationInputApplyList",
    "TotalBidMatrixCalculationInputFields",
    "TotalBidMatrixCalculationInputTextFields",
    "TotalBidMatrixCalculationOutput",
    "TotalBidMatrixCalculationOutputGraphQL",
    "TotalBidMatrixCalculationOutputWrite",
    "TotalBidMatrixCalculationOutputApply",
    "TotalBidMatrixCalculationOutputList",
    "TotalBidMatrixCalculationOutputWriteList",
    "TotalBidMatrixCalculationOutputApplyList",
    "TotalBidMatrixCalculationOutputFields",
    "TotalBidMatrixCalculationOutputTextFields",
    "TurbineEfficiencyCurve",
    "TurbineEfficiencyCurveGraphQL",
    "TurbineEfficiencyCurveWrite",
    "TurbineEfficiencyCurveApply",
    "TurbineEfficiencyCurveList",
    "TurbineEfficiencyCurveWriteList",
    "TurbineEfficiencyCurveApplyList",
    "TurbineEfficiencyCurveFields",
    "WaterValueBasedPartialBidConfiguration",
    "WaterValueBasedPartialBidConfigurationGraphQL",
    "WaterValueBasedPartialBidConfigurationWrite",
    "WaterValueBasedPartialBidConfigurationApply",
    "WaterValueBasedPartialBidConfigurationList",
    "WaterValueBasedPartialBidConfigurationWriteList",
    "WaterValueBasedPartialBidConfigurationApplyList",
    "WaterValueBasedPartialBidConfigurationFields",
    "WaterValueBasedPartialBidConfigurationTextFields",
    "WaterValueBasedPartialBidMatrixCalculationInput",
    "WaterValueBasedPartialBidMatrixCalculationInputGraphQL",
    "WaterValueBasedPartialBidMatrixCalculationInputWrite",
    "WaterValueBasedPartialBidMatrixCalculationInputApply",
    "WaterValueBasedPartialBidMatrixCalculationInputList",
    "WaterValueBasedPartialBidMatrixCalculationInputWriteList",
    "WaterValueBasedPartialBidMatrixCalculationInputApplyList",
    "WaterValueBasedPartialBidMatrixCalculationInputFields",
    "WaterValueBasedPartialBidMatrixCalculationInputTextFields",
    "Watercourse",
    "WatercourseGraphQL",
    "WatercourseWrite",
    "WatercourseApply",
    "WatercourseList",
    "WatercourseWriteList",
    "WatercourseApplyList",
    "WatercourseFields",
    "WatercourseTextFields",
]
